c                               TOPIC V (1)
c MODULAR PROGRAMMING
c*********************************************************************************
c Modules are like intrinsic functions (e.g. SQRT, etc) which are procedures
c we use in the progression of the program yet we need not worry about how
c they handle the details of their task. But unlike intrinsic functions,
c modules are procedures defined by the programmer and packaged, ready for use
c For example if you are making an Apple-pie you can either make your own
c crust or buy it from the store and add the filling. Obviously one
c is more conveneient especially if the module you need already exists.


c This program illustrates SUBROUTINE "modules" in Fortran:
c Subroutine modules are invoked using the CALL statement.
c The CALL statement provides the module with some input data(variables and
c constants) and returns one or more output data(variables) such that the
c details of the operation is hidden from the programmer.
c the list of the input and output data is called the ARGUMENT LIST. Each
c variable or a constant is called an ARGUMENT of the CALL statement.
c note that if an input argument is modified in a module then its new value
c will be returned and becomes effective in the caller program.
c*********************************************************************************



c This program converts Cartesian (X,Y) to Polar(R,THETA)  coordinates.
c This part of the program is called the MAIN module; it acts like a 
c central command control and it only directs the oparation without
c doing much of the detailed work itself.

        REAL X,Y,R,THETA

        CALL GETXY(X,Y)          !subroutine gets the input data
                                ! (X,Y) is the argument list both returned
                                ! no input to GETXY is needed.

        CALL DOCALC(X,Y,R,THETA)  !subroutine does the calculation
                                  !X and Y are input and R and THETA are returned

        CALL OUTPUT(R,THETA)   !subroutine outputs the data returned by DOCALC
                               !R and THETA are input arguments 
        END

c So the input and output modules can be as sophisticaed as one wants and can 
c be develoeped and tested separately.

c=========================================================================
c the above line is a separation marker between the MAIN and
c subroutines.
c This subroutine-module does the calculation only

c Note that subroutines need NOT be listed in the order they are called.
c Also that the module is like an INDEPENDENT LITTLE PROGRAM IN ITSELF,
c so it can have its own naming of variables, as long as they correspond
c correctly(in order of appearance in the argument list) to those of the
c caller. In fact it is the responsibility of the caller (MAIN or another
c subroutine) to conform to the argument listings of a subroutine.

        SUBROUTINE DOCALC(X,Y,R,T)
        REAL X,Y,R,T              !need independent declarations for each Subr.
        R=SQRT(X**2 + Y**2)
        T=ATAN(Y/X)               !will be returned in Theta in MAIN
        RETURN                    !w/o this no data will be passed back
        END                       !end each subr like any Fortran routine

c==========================================================================
c This subroutine-module outputs two real numbers to the console.
c Note the generic form of this and the next subroutine. This means that
c they may have usage in another program which may need the same functions
c and services. This allows for code reuse and increased productivity.

        SUBROUTINE OUTPUT(A,B)
        REAL A,B
        WRITE(*,*)           ! allow a little space
        WRITE(*,*)'The resulting number''s are:'  !double quotes become
        WRITE(*,*)A,' and ',B                    !single on output
        RETURN                         !RETURN means go back to the caller
        END                          !END marks the end of the subroutine
                                     !as with any Fortran program
c=========================================================================
c Generic subroutine to get two real numbers
        
        SUBROUTINE GETXY(X,X)
        REAL X,Y
        WRITE(*,*)'Please enter 2 numbers'
        READ(*,*)X,Y
        RETURN                   
        END                      
                                 
c=========================================================================
c The benefits of modularization can't be over-estimated        
c 1. Easier programming: a. design  b. testing
c 2. Readability
c 3. Each module is INDEPENDENT: so,
c               A. Team writing is possible
c               B. CODE REUSE becomes possible




c ======= * ======== * ========= * ======= * ======== * ========= 
c REGISTER YOUR OWNERSHIP TO GET THE VIDEO TAPE & EXTRA LESSONS 
c IN TIME FOR WHEN YOU FINISH THE TOPICS PROVIDED HERE
c ======= * ======== * ========= * ======= * ======== * ========= 

